/*
 * Copyright (c) 2008 Helder Correia
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
*/

#include "colorwizard.h"
#include "window.h"

#include <QAction>
#include <QApplication>
#include <QColorDialog>
#include <QMenu>
#include <QMenuBar>
#include <QWidget>

Window::Window(QWidget *parent)
    : QMainWindow(parent)
{
    QWidget *centralWidget = new QWidget(this);
    setCentralWidget(centralWidget);
    ui = new Ui::AutoHilite;
    ui->setupUi(centralWidget);

    QMenuBar *bar = menuBar();
    QMenu *schemes = bar->addMenu(tr("&Schemes"));

    QAction *scheme1 = new QAction(tr("Black on white"),     this);
    QAction *scheme2 = new QAction(tr("Solaris"),            this);
    QAction *scheme3 = new QAction(tr("CDE"),                this);
    QAction *scheme4 = new QAction(tr("Digital CDE"),        this);
    QAction *scheme5 = new QAction(tr("Green on black"),     this);
    QAction *scheme6 = new QAction(tr("White on black"),     this);
    QAction *scheme7 = new QAction(tr("White on dark blue"), this);
    QAction *scheme8 = new QAction(tr("Yellow on blue"),     this);

    schemes->addAction(scheme1);
    schemes->addAction(scheme2);
    schemes->addAction(scheme3);
    schemes->addAction(scheme4);
    schemes->addAction(scheme5);
    schemes->addAction(scheme6);
    schemes->addAction(scheme7);
    schemes->addAction(scheme8);

    connect(scheme1, SIGNAL(triggered()), SLOT(setScheme1()));
    connect(scheme2, SIGNAL(triggered()), SLOT(setScheme2()));
    connect(scheme3, SIGNAL(triggered()), SLOT(setScheme3()));
    connect(scheme4, SIGNAL(triggered()), SLOT(setScheme4()));
    connect(scheme5, SIGNAL(triggered()), SLOT(setScheme5()));
    connect(scheme6, SIGNAL(triggered()), SLOT(setScheme6()));
    connect(scheme7, SIGNAL(triggered()), SLOT(setScheme7()));
    connect(scheme8, SIGNAL(triggered()), SLOT(setScheme8()));

    connect(ui->bgButton, SIGNAL(clicked()),
             this, SLOT(chooseBg()));
    connect(ui->fgButton, SIGNAL(clicked()),
             this, SLOT(chooseFg()));

    connect(ui->fgRedSpin, SIGNAL(valueChanged(int)),
             this, SLOT(findColors()));
    connect(ui->fgGreenSpin, SIGNAL(valueChanged(int)),
             this, SLOT(findColors()));
    connect(ui->fgBlueSpin, SIGNAL(valueChanged(int)),
             this, SLOT(findColors()));

    connect(ui->bgRedSpin, SIGNAL(valueChanged(int)),
             this, SLOT(findColors()));
    connect(ui->bgGreenSpin, SIGNAL(valueChanged(int)),
             this, SLOT(findColors()));
    connect(ui->bgBlueSpin, SIGNAL(valueChanged(int)),
             this, SLOT(findColors()));

    connect(this, SIGNAL(colorChanged()), SLOT(findColors()));

    setScheme1();
}

void Window::chooseBg()
{
    QColor c = QColorDialog::getColor();
    if (c.isValid())
    {
        setBgColor(c);
        emit colorChanged();
    }
}

void Window::chooseFg()
{
    QColor c = QColorDialog::getColor();
    if (c.isValid())
    {
        setFgColor(c);
        emit colorChanged();
    }
}

void Window::findColors()
{
    int r, g, b;

    r = ui->bgRedSpin->value();
    g = ui->bgGreenSpin->value();
    b = ui->bgBlueSpin->value();
    QColor bg(r, g, b);
    QBrush bgBrush(bg);

    r = ui->fgRedSpin->value();
    g = ui->fgGreenSpin->value();
    b = ui->fgBlueSpin->value();
    QColor fg(r, g, b);
    QBrush fgBrush(fg);

    QVector<QColor> vec = ColorWizard::highlight(bg, fg, 3);
    QBrush brush1(vec.at(0));
    QBrush brush2(vec.at(1));
    QBrush brush3(vec.at(2));

    QPalette pal;

    pal.setBrush(QPalette::Button, bgBrush);
    ui->bgButton->setPalette(pal);
    pal.setBrush(QPalette::Button, fgBrush);
    ui->fgButton->setPalette(pal);

    pal.setBrush(QPalette::Window, bgBrush);
    ui->hiliteBgFrame->setPalette(pal);

    pal.setBrush(QPalette::Window, brush1);
    ui->hilite1->setPalette(pal);
    pal.setBrush(QPalette::Window, brush2);
    ui->hilite2->setPalette(pal);
    pal.setBrush(QPalette::Window, brush3);
    ui->hilite3->setPalette(pal);
}

void Window::setBgColor(const QColor &c)
{
    ui->bgRedSpin->blockSignals(true);
    ui->bgGreenSpin->blockSignals(true);
    ui->bgBlueSpin->blockSignals(true);

    ui->bgRedSpin->setValue(c.red());
    ui->bgGreenSpin->setValue(c.green());
    ui->bgBlueSpin->setValue(c.blue());

    ui->bgRedSpin->blockSignals(false);
    ui->bgGreenSpin->blockSignals(false);
    ui->bgBlueSpin->blockSignals(false);
}

void Window::setFgColor(const QColor &c)
{
    ui->fgRedSpin->blockSignals(true);
    ui->fgGreenSpin->blockSignals(true);
    ui->fgBlueSpin->blockSignals(true);

    ui->fgRedSpin->setValue(c.red());
    ui->fgGreenSpin->setValue(c.green());
    ui->fgBlueSpin->setValue(c.blue());

    ui->fgRedSpin->blockSignals(false);
    ui->fgGreenSpin->blockSignals(false);
    ui->fgBlueSpin->blockSignals(false);
}

void Window::setScheme1()
{
    QColor bg(255, 255, 255);
    QColor fg(0, 0, 0);
    setBgColor(bg);
    setFgColor(fg);
    emit colorChanged();
}

void Window::setScheme2()
{
    QColor bg(147, 151, 165);
    QColor fg(0, 0, 0);
    setBgColor(bg);
    setFgColor(fg);
    emit colorChanged();
}

void Window::setScheme3()
{
    QColor bg(129, 129, 129);
    QColor fg(255, 255, 255);
    setBgColor(bg);
    setFgColor(fg);
    emit colorChanged();
}

void Window::setScheme4()
{
    QColor bg(55, 77, 78);
    QColor fg(255, 255, 255);
    setBgColor(bg);
    setFgColor(fg);
    emit colorChanged();
}

void Window::setScheme5()
{
    QColor bg(0, 0, 0);
    QColor fg(0, 255, 0);
    setBgColor(bg);
    setFgColor(fg);
    emit colorChanged();
}

void Window::setScheme6()
{
    QColor bg(0, 0, 0);
    QColor fg(255, 255, 255);
    setBgColor(bg);
    setFgColor(fg);
    emit colorChanged();
}

void Window::setScheme7()
{
    QColor bg(0, 42, 78);
    QColor fg(255, 255, 255);
    setBgColor(bg);
    setFgColor(fg);
    emit colorChanged();
}

void Window::setScheme8()
{
    QColor bg(0, 0, 255);
    QColor fg(255, 255, 0);
    setBgColor(bg);
    setFgColor(fg);
    emit colorChanged();
}
